@@ -22,7 +22,7 @@ Test the query while importing at the same time. This was raising
 some SQLITE_MISUSED errors before.
 """
 import os, dbus
-import gobject
+from gi.repository import GObject
 from dbus.mainloop.glib import DBusGMainLoop
 
 from common.utils import configuration as cfg
@@ -35,7 +35,7 @@ class TestSqliteMisused (CommonTrackerStoreTest):
     Send queries while importing files (in .ttl directory)
     """
     def setUp (self):
-        self.main_loop = gobject.MainLoop ()
+        self.main_loop = GObject.MainLoop ()
         self.files_counter = 0
         
     def test_queries_while_import (self):
@@ -49,9 +49,9 @@ class TestSqliteMisused (CommonTrackerStoreTest):
                                      reply_handler=self.loaded_success_cb,
                                      error_handler=self.loaded_failed_cb)
         
-        gobject.timeout_add_seconds (2, self.run_a_query)
+        GObject.timeout_add_seconds (2, self.run_a_query)
         # Safeguard of 60 seconds. The last reply should quit the loop
-        gobject.timeout_add_seconds (60, self.timeout_cb)
+        GObject.timeout_add_seconds (60, self.timeout_cb)
         self.main_loop.run ()
 
     def run_a_query (self):
